@@ -1,5 +1,6 @@
 use crate::relay::RelayWrapper;
 use crate::stats::Stats;
+use crate::Untyped;
 use crate::{
     envelop::{IntoBoxedMessage, TypeTag},
     error::{GenericError, SendError, StdSyncSendError},
@@ -13,12 +14,15 @@ use core::{
     mem,
     pin::Pin,
     sync::atomic::{AtomicBool, AtomicI64, Ordering},
-    task::{Context, Poll},
 };
-use futures::{future::poll_fn, Future, FutureExt};
+use futures::{pin_mut, Stream};
+use futures::{Future, FutureExt, StreamExt};
 use std::{borrow::Cow, sync::Arc};
 use tokio::sync::{oneshot, Notify};
 
+pub type BusPollerCallback = Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>;
+pub type UntypedPollerCallback = Box<dyn FnOnce(Untyped) -> BusPollerCallback>;
+
 struct SlabCfg;
 impl sharded_slab::Config for SlabCfg {
     const RESERVED_BITS: usize = 1;
@@ -48,15 +52,15 @@ where
     M: Message,
     E: StdSyncSendError,
 {
-    fn poll_events(&self, ctx: &mut Context<'_>, bus: &Bus) -> Poll<Event<M, E>>;
+    type Stream: Stream<Item = Event<M, E>> + Send;
+
+    fn event_stream(&self, bus: Bus) -> Self::Stream;
 }
 
 pub trait ReciveUntypedReceiver: Sync {
-    fn poll_events(
-        &self,
-        ctx: &mut Context<'_>,
-        bus: &Bus,
-    ) -> Poll<Event<Box<dyn Message>, GenericError>>;
+    type Stream: Stream<Item = Event<Box<dyn Message>, GenericError>> + Send;
+
+    fn event_stream(&self, bus: Bus) -> Self::Stream;
 }
 
 pub trait WrapperReturnTypeOnly<R: Message>: Send + Sync {
@@ -74,9 +78,7 @@ pub trait WrapperErrorTypeOnly<E: StdSyncSendError>: Send + Sync {
 }
 
 pub trait WrapperReturnTypeAndError<R: Message, E: StdSyncSendError>: Send + Sync {
-    fn start_polling_events(
-        self: Arc<Self>,
-    ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>;
+    fn start_polling_events(self: Arc<Self>) -> BusPollerCallback;
     fn add_response_listener(
         &self,
         listener: oneshot::Sender<Result<R, Error<(), E>>>,
@@ -123,9 +125,7 @@ pub trait ReceiverTrait: TypeTagAccept + Send + Sync {
     fn reserve_notify(&self, tt: &TypeTag) -> Arc<Notify>;
     fn increment_processing(&self, tt: &TypeTag);
 
-    fn start_polling(
-        self: Arc<Self>,
-    ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>;
+    fn start_polling(self: Arc<Self>) -> BusPollerCallback;
 }
 
 pub trait ReceiverPollerBuilder {
@@ -181,15 +181,20 @@ where
     E: StdSyncSendError,
     S: SendUntypedReceiver + ReciveTypedReceiver<R, E> + Send + Sync + 'static,
 {
-    fn start_polling_events(
-        self: Arc<Self>,
-    ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>> {
+    fn start_polling_events(self: Arc<Self>) -> BusPollerCallback {
         Box::new(move |bus| {
             Box::pin(async move {
+                let this = self.clone();
+                let events = this.inner.event_stream(bus);
+                pin_mut!(events);
+
                 loop {
-                    let this = self.clone();
-                    let bus = bus.clone();
-                    let event = poll_fn(move |ctx| this.inner.poll_events(ctx, &bus)).await;
+                    let event = if let Some(event) = events.next().await {
+                        event
+                    } else {
+                        self.context.closed.notify_waiters();
+                        break;
+                    };
 
                     match event {
                         Event::Error(err) => error!("Batch Error: {}", err),
@@ -246,7 +251,7 @@ where
         Ok(self
             .waiters
             .insert(Waiter::WithErrorType(listener))
-            .ok_or_else(|| Error::AddListenerError)? as _)
+            .ok_or(Error::AddListenerError)? as _)
     }
 
     fn response(&self, mid: u64, resp: Result<R, Error<(), E>>) -> Result<(), Error> {
@@ -284,7 +289,7 @@ where
         Ok(self
             .waiters
             .insert(Waiter::WithoutErrorType(listener))
-            .ok_or_else(|| Error::AddListenerError)? as _)
+            .ok_or(Error::AddListenerError)? as _)
     }
 }
 
@@ -302,7 +307,7 @@ where
         Ok(self
             .waiters
             .insert(Waiter::BoxedWithError(listener))
-            .ok_or_else(|| Error::AddListenerError)? as _)
+            .ok_or(Error::AddListenerError)? as _)
     }
 }
 
@@ -365,8 +370,8 @@ where
             .downcast::<M>()
             .map_err(|_| Error::MessageCastError)?;
 
-        Ok(SendTypedReceiver::send(&self.inner, mid, *boxed, req, bus)
-            .map_err(|err| Error::from(err.into_boxed()))?)
+        SendTypedReceiver::send(&self.inner, mid, *boxed, req, bus)
+            .map_err(|err| Error::from(err.into_boxed()))
     }
 
     fn stats(&self) -> Stats {
@@ -410,7 +415,7 @@ where
         Ok(self
             .waiters
             .insert(Waiter::Boxed(listener))
-            .ok_or_else(|| Error::AddListenerError)? as _)
+            .ok_or(Error::AddListenerError)? as _)
     }
 
     fn ready_notify(&self) -> &Notify {
@@ -458,9 +463,7 @@ where
         self.context.response.clone()
     }
 
-    fn start_polling(
-        self: Arc<Self>,
-    ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>> {
+    fn start_polling(self: Arc<Self>) -> BusPollerCallback {
         self.start_polling_events()
     }
 
@@ -489,12 +492,10 @@ pub struct AnyReceiver<'a> {
 }
 
 impl<'a> AnyReceiver<'a> {
-    pub fn new<M, R, E, S>(rcvr: &'a S) -> Self
+    pub fn new<M, S>(rcvr: &'a S) -> Self
     where
         M: Message,
-        R: Message,
-        E: StdSyncSendError,
-        S: SendTypedReceiver<M> + ReciveTypedReceiver<R, E> + 'static,
+        S: SendTypedReceiver<M> + 'static,
     {
         let send_typed_receiver = rcvr as &(dyn SendTypedReceiver<M>);
         let send_typed_receiver: TraitObject = unsafe { mem::transmute(send_typed_receiver) };
@@ -835,9 +836,7 @@ impl Receiver {
     }
 
     #[inline]
-    pub fn start_polling(
-        &self,
-    ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>> {
+    pub fn start_polling(&self) -> BusPollerCallback {
         self.inner.clone().start_polling()
     }
 
