@@ -4,21 +4,23 @@ use std::{
         atomic::{AtomicU64, Ordering},
         Arc,
     },
-    task::{Context, Poll},
 };
 
 use crate::{
     buffer_unordered_poller_macro,
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
-    receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
+    receiver::{
+        Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver,
+        UntypedPollerCallback,
+    },
     receivers::Request,
     AsyncHandler, Bus, Message, Untyped,
 };
 
 use super::{BufferUnorderedConfig, BufferUnorderedStats};
 
-use futures::Future;
+use futures::{Future, Stream};
 use parking_lot::Mutex;
 use tokio::sync::mpsc::{self, UnboundedSender};
 
@@ -45,7 +47,7 @@ where
 {
     tx: mpsc::UnboundedSender<Request<M>>,
     stats: Arc<BufferUnorderedStats>,
-    srx: Mutex<mpsc::UnboundedReceiver<Event<R, E>>>,
+    srx: Mutex<Option<mpsc::UnboundedReceiver<Event<R, E>>>>,
 }
 
 impl<T, M, R, E> ReceiverSubscriberBuilder<T, M, R, E> for BufferUnorderedAsync<M, R, E>
@@ -57,14 +59,7 @@ where
 {
     type Config = BufferUnorderedConfig;
 
-    fn build(
-        cfg: Self::Config,
-    ) -> (
-        Self,
-        Box<
-            dyn FnOnce(Untyped) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
-        >,
-    ) {
+    fn build(cfg: Self::Config) -> (Self, UntypedPollerCallback) {
         let stats = Arc::new(BufferUnorderedStats {
             buffer: AtomicU64::new(0),
             buffer_total: AtomicU64::new(cfg.buffer_size as _),
@@ -93,7 +88,7 @@ where
             BufferUnorderedAsync::<M, R, E> {
                 tx,
                 stats,
-                srx: Mutex::new(srx),
+                srx: Mutex::new(Some(srx)),
             },
             poller,
         )
@@ -140,12 +135,11 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn poll_events(&self, ctx: &mut Context<'_>, _bus: &Bus) -> Poll<Event<R, E>> {
-        let poll = self.srx.lock().poll_recv(ctx);
-        match poll {
-            Poll::Pending => Poll::Pending,
-            Poll::Ready(Some(event)) => Poll::Ready(event),
-            Poll::Ready(None) => Poll::Ready(Event::Exited),
-        }
+    type Stream = Pin<Box<dyn Stream<Item = Event<R, E>> + Send>>;
+
+    fn event_stream(&self, _: Bus) -> Self::Stream {
+        let mut rx = self.srx.lock().take().unwrap();
+
+        Box::pin(futures::stream::poll_fn(move |cx| rx.poll_recv(cx)))
     }
 }
