@@ -1,20 +1,19 @@
-use std::{
-    pin::Pin,
-    sync::Arc,
-    task::{Context, Poll},
-};
+use std::{pin::Pin, sync::Arc};
 
 use crate::{
     batch_synchronized_poller_macro,
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
-    receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
+    receiver::{
+        Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver,
+        UntypedPollerCallback,
+    },
     receivers::Request,
     BatchSynchronizedHandler, Bus, Message, Untyped,
 };
 
 use super::SynchronizedBatchedConfig;
-use futures::{executor::block_on, Future};
+use futures::{executor::block_on, Future, Stream};
 use tokio::sync::{
     mpsc::{self, UnboundedSender},
     Mutex,
@@ -44,7 +43,7 @@ where
     E: StdSyncSendError,
 {
     tx: mpsc::UnboundedSender<Request<M>>,
-    srx: parking_lot::Mutex<mpsc::UnboundedReceiver<Event<R, E>>>,
+    srx: parking_lot::Mutex<Option<mpsc::UnboundedReceiver<Event<R, E>>>>,
 }
 
 impl<T, M, R> ReceiverSubscriberBuilder<T, M, R, T::Error>
@@ -57,14 +56,7 @@ where
 {
     type Config = SynchronizedBatchedConfig;
 
-    fn build(
-        cfg: Self::Config,
-    ) -> (
-        Self,
-        Box<
-            dyn FnOnce(Untyped) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
-        >,
-    ) {
+    fn build(cfg: Self::Config) -> (Self, UntypedPollerCallback) {
         let (stx, srx) = mpsc::unbounded_channel();
         let (tx, rx) = mpsc::unbounded_channel();
 
@@ -78,7 +70,7 @@ where
         (
             SynchronizedBatchedSync::<M, R, T::Error> {
                 tx,
-                srx: parking_lot::Mutex::new(srx),
+                srx: parking_lot::Mutex::new(Some(srx)),
             },
             poller,
         )
@@ -121,12 +113,11 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn poll_events(&self, ctx: &mut Context<'_>, _bus: &Bus) -> Poll<Event<R, E>> {
-        let poll = self.srx.lock().poll_recv(ctx);
-        match poll {
-            Poll::Pending => Poll::Pending,
-            Poll::Ready(Some(event)) => Poll::Ready(event),
-            Poll::Ready(None) => Poll::Ready(Event::Exited),
-        }
+    type Stream = Pin<Box<dyn Stream<Item = Event<R, E>> + Send>>;
+
+    fn event_stream(&self, _: Bus) -> Self::Stream {
+        let mut rx = self.srx.lock().take().unwrap();
+
+        Box::pin(futures::stream::poll_fn(move |cx| rx.poll_recv(cx)))
     }
 }
