@@ -1,11 +1,7 @@
-use std::{
-    pin::Pin,
-    sync::Arc,
-    task::{Context, Poll},
-};
+use std::{pin::Pin, sync::Arc};
 
-use crate::synchronized_poller_macro;
-use futures::{executor::block_on, Future};
+use crate::{receiver::UntypedPollerCallback, synchronized_poller_macro};
+use futures::{executor::block_on, Future, Stream};
 
 use super::SynchronizedConfig;
 use crate::{
@@ -45,7 +41,7 @@ where
     E: StdSyncSendError,
 {
     tx: mpsc::UnboundedSender<Request<M>>,
-    srx: parking_lot::Mutex<mpsc::UnboundedReceiver<Event<R, E>>>,
+    srx: parking_lot::Mutex<Option<mpsc::UnboundedReceiver<Event<R, E>>>>,
 }
 
 impl<T, M, R, E> ReceiverSubscriberBuilder<T, M, R, E> for SynchronizedSync<M, R, E>
@@ -57,14 +53,7 @@ where
 {
     type Config = SynchronizedConfig;
 
-    fn build(
-        _cfg: Self::Config,
-    ) -> (
-        Self,
-        Box<
-            dyn FnOnce(Untyped) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>,
-        >,
-    ) {
+    fn build(_cfg: Self::Config) -> (Self, UntypedPollerCallback) {
         let (stx, srx) = mpsc::unbounded_channel();
         let (tx, rx) = mpsc::unbounded_channel();
 
@@ -78,7 +67,7 @@ where
         (
             SynchronizedSync::<M, R, E> {
                 tx,
-                srx: parking_lot::Mutex::new(srx),
+                srx: parking_lot::Mutex::new(Some(srx)),
             },
             poller,
         )
@@ -121,12 +110,11 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn poll_events(&self, ctx: &mut Context<'_>, _bus: &Bus) -> Poll<Event<R, E>> {
-        let poll = self.srx.lock().poll_recv(ctx);
-        match poll {
-            Poll::Pending => Poll::Pending,
-            Poll::Ready(Some(event)) => Poll::Ready(event),
-            Poll::Ready(None) => Poll::Ready(Event::Exited),
-        }
+    type Stream = Pin<Box<dyn Stream<Item = Event<R, E>> + Send>>;
+
+    fn event_stream(&self, _: Bus) -> Self::Stream {
+        let mut rx = self.srx.lock().take().unwrap();
+
+        Box::pin(futures::stream::poll_fn(move |cx| rx.poll_recv(cx)))
     }
 }
