@@ -1,6 +1,7 @@
-use std::task::{Context, Poll};
+use std::pin::Pin;
 
 use async_trait::async_trait;
+use futures::Stream;
 use messagebus::{
     derive::{Error as MbError, Message},
     error::{self, GenericError},
@@ -44,9 +45,14 @@ impl AsyncHandler<Msg<i32>> for TmpReceiver {
     }
 }
 
+pub type TestRelayRxChannelCell =
+    Mutex<Option<mpsc::UnboundedReceiver<Event<Box<dyn Message>, GenericError>>>>;
+pub type TestRelayRxStream =
+    Pin<Box<dyn Stream<Item = Event<Box<dyn Message>, error::GenericError>> + Send>>;
+
 pub struct TestRelay {
     stx: mpsc::UnboundedSender<Event<Box<dyn Message>, GenericError>>,
-    srx: Mutex<mpsc::UnboundedReceiver<Event<Box<dyn Message>, GenericError>>>,
+    srx: TestRelayRxChannelCell,
 }
 
 impl TypeTagAccept for TestRelay {
@@ -103,9 +109,7 @@ impl TypeTagAccept for TestRelay {
             &Msg::<i32>::type_tag_(),
             &Msg::<i64>::type_tag_(),
             &Error::type_tag_(),
-        ) {
-            return;
-        }
+        ) {}
     }
 }
 
@@ -154,17 +158,12 @@ impl SendUntypedReceiver for TestRelay {
 }
 
 impl ReciveUntypedReceiver for TestRelay {
-    fn poll_events(
-        &self,
-        ctx: &mut Context<'_>,
-        _bus: &Bus,
-    ) -> Poll<Event<Box<dyn Message>, error::GenericError>> {
-        let poll = self.srx.lock().poll_recv(ctx);
-        match poll {
-            Poll::Pending => Poll::Pending,
-            Poll::Ready(Some(event)) => Poll::Ready(event),
-            Poll::Ready(None) => Poll::Ready(Event::Exited),
-        }
+    type Stream = TestRelayRxStream;
+
+    fn event_stream(&self) -> Self::Stream {
+        let mut rx = self.srx.lock().take().unwrap();
+
+        Box::pin(futures::stream::poll_fn(move |cx| rx.poll_recv(cx)))
     }
 }
 
@@ -173,7 +172,7 @@ async fn test_relay() {
     let (stx, srx) = mpsc::unbounded_channel();
     let relay = TestRelay {
         stx,
-        srx: Mutex::new(srx),
+        srx: Mutex::new(Some(srx)),
     };
 
     let (b, poller) = Bus::build()
